package de.kleopatra.forms.examples;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Arrays;

import javax.swing.*;

import com.jgoodies.binding.value.ValueModel;

import de.kleopatra.forms.examples.data.*;
import de.kleopatra.forms.view.builder.FormFactory;
import de.kleopatra.forms.view.builder.FormUtils;
import de.kleopatra.support.KleoUtilities;
import de.kleopatra.view.binding.BeanBinding;

/**
 * Edit bean properties.
 *
 *
 * @author (C) 2003, 2004 Jeanette Winzenburg, Berlin
 * @version $Revision: 1.5 $ - $Date: 2004/10/04 11:56:04 $
 *
 */
public class SimpleBinding {

  // resourcenames
  private static final String SIMPLE_RESOURCE = "simplebinding";
  private BeanBinding binding;
  private JComponent form;

//-------------------- public access

  /** returns the JComponent associated with the resource.
   *  PRE: getResource() != null.
   */
  public JComponent getFormComponent() {
    if (form == null) {
      form = buildForm(SIMPLE_RESOURCE);
      bind(form);
      configure(form);
    }
    return form;
  }

//------------------------ build
  private JComponent buildForm(String resource) {
    JComponent comp = (JComponent) FormFactory.getFormLoader().load(resource,
                        getClass());
    return comp;
  }

  //--------------------- bind
  private void bind(JComponent form) {

    // bind the properties of a DataObject
    binding = new BeanBinding(form, null, true);

    // simple text
    binding.bindTextInput("name");
    binding.bindTextInput("phone");

    // non-editable text
    binding.bindTextInput("countryCode", false);

    // spinner 
    binding.bindSpinnerInput("age", new SpinnerNumberModel(40, 40, 100, 1));

    // comboBox: selection bound to bean property
    // available options as ListModel, editable
    binding.bindComboBoxInput("country", CountryCodeResolver.countriesModel,
        true);

    // read-only boolean property
    binding.bindBooleanInput("reachable");
    binding.setEnabled("reachable", false);

    // bind enable of "phone" widget to selection state of "reachable"
    binding.addToggleDependents("reachable",
        Arrays.asList(new String[]{ "phone" }));
    binding.bindTextAreaInput("notes");

    // several radiobuttons bound to a bean property
    binding.bindRadioGroup(Arrays.asList(DataObject.PROFESSION_IDS),
        "profession",
        Arrays.asList(DataObject.PROFESSIONS));

    // bind the controller
    bindController(form);
  }

  /** bind the overview list.
   *
   * @param comp
   */
  protected void bindController(JComponent comp) {
    ValueModel overview = new SampleController();
    BeanBinding controller = new BeanBinding(comp, overview, false);
    controller.bindListAndSelectionInput("dataList", "dataList", "value");
    overview.addValueChangeListener(createOverviewListener());
  }

  private PropertyChangeListener createOverviewListener() {
    PropertyChangeListener l = new PropertyChangeListener() {

      public void propertyChange(PropertyChangeEvent evt) {
        binding.setTarget(evt.getNewValue());
      }
    };
    return l;
  }

//----------------------- configure
  private void configure(JComponent container) {
    JList list = KleoUtilities.findList(container, "dataList");
    list.setCellRenderer(TutorialUtils.createDataObjectListCellRenderer());
  }

//---------------------------init ui
  private static void initUI() {

    // set the LF
    System.getProperties().setProperty("swing.defaultlaf",
        "com.jgoodies.plaf.plastic.PlasticLookAndFeel");
  }

//---------------------------Main

  public static void main(String[] args) {
    initUI();
    SimpleBinding form = new SimpleBinding();
    FormUtils.createFrame(form.getFormComponent(), "Simple Binding", true);
  }
}
