package de.kleopatra.forms.examples;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Arrays;

import javax.swing.*;

import com.jgoodies.binding.value.ValueModel;

import de.kleopatra.forms.examples.data.*;
import de.kleopatra.forms.view.builder.FormUtils;
import de.kleopatra.support.KleoUtilities;
import de.kleopatra.view.binding.AbstractBoundForm;
import de.kleopatra.view.binding.BeanBinding;

/**
 * Edit bean properties.
 *
 * Same as SimpleBinding - but extending AbstractBoundForm.
 *
 * @author (C) 2003, 2004 Jeanette Winzenburg, Berlin
 * @version $Revision: 1.4 $ - $Date: 2004/10/04 11:56:04 $
 */
public class SimpleBindingForm extends AbstractBoundForm {

  // resourcenames
  private static final String SIMPLE_RESOURCE = "simplebinding";

//------------------ overriding super's public getters
  public String getResource() {
    return SIMPLE_RESOURCE;
  }

//--------------------- bind 
  protected void bind(JComponent form) {

    // bind the properties of a DataObject
    binding = new BeanBinding(form, getTarget(), true);

    // simple text
    binding.bindTextInput("name");
    binding.bindTextInput("phone");

    // non-editable text
    binding.bindTextInput("countryCode", false);

    // spinner 
    binding.bindSpinnerInput("age", new SpinnerNumberModel(40, 40, 100, 1));

    // comboBox: selection bound to bean property
    // available options as ListModel, editable
    binding.bindComboBoxInput("country", CountryCodeResolver.countriesModel,
        true);

    // read-only boolean property
    binding.bindBooleanInput("reachable");
    binding.setEnabled("reachable", false);

    // bind enable of "phone" widget to selection state of "reachable"
    binding.addToggleDependents("reachable",
        Arrays.asList(new String[]{ "phone" }));
    binding.bindTextAreaInput("notes");

    // several radiobuttons bound to a bean property
    binding.bindRadioGroup(Arrays.asList(DataObject.PROFESSION_IDS),
        "profession",
        Arrays.asList(DataObject.PROFESSIONS));

    // bind the controller
    bindController(form);
  }

  /** bind the overview list.
   *
   * @param comp
   */
  protected void bindController(JComponent comp) {
    ValueModel overview = new SampleController();
    BeanBinding controller = new BeanBinding(comp, overview, false);
    controller.bindListAndSelectionInput("dataList", "dataList", "value");
    overview.addValueChangeListener(createOverviewListener());
  }

  private PropertyChangeListener createOverviewListener() {
    PropertyChangeListener l = new PropertyChangeListener() {

      public void propertyChange(PropertyChangeEvent evt) {
        setTarget(evt.getNewValue());
      }
    };
    return l;
  }

//----------------------- configure
  protected void configure(JComponent container) {
    JList list = KleoUtilities.findList(container, "dataList");
    list.setCellRenderer(TutorialUtils.createDataObjectListCellRenderer());
  }

//---------------------------init ui
  private static void initUI() {

    // set the default lf property
    System.getProperties().setProperty("swing.defaultlaf",
        "com.jgoodies.plaf.plastic.PlasticLookAndFeel");
  }

//---------------------------Main

  public static void main(String[] args) {
    initUI();
    SimpleBindingForm form = new SimpleBindingForm();
    FormUtils.createFrame(form, "Simple Binding Form", true);
  }
}
