package de.kleopatra.forms.examples;

import java.awt.Container;
import java.awt.Window;
import java.util.Arrays;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.SwingUtilities;

import com.jgoodies.binding.beans.Model;

import de.kleopatra.forms.view.builder.*;
import de.kleopatra.view.binding.AbstractBoundForm;
import de.kleopatra.view.binding.BeanBinding;
import de.kleopatra.view.presentation.ActionFactory;

/** Example how to use a given layout/presentation description.
 *  <p>
 *  Switches application layout on the
 *  fly (toggle between buttonBar and buttonStack).
 *  Uses binding to example data.
 *
 * @author (C) 2003 Jeanette Winzenburg, Berlin
 * @version $Revision: 1.3 $ - $Date: 2004/10/04 11:56:04 $
 *
 */
public class SwitchFormLayoutWB extends AbstractBoundForm {

// resourcenames
  String applicationCenterID = "mainText";
  String applicationButtonID = "buttonpanel";

// scheme for center/south arrangement
  LayoutScheme verticalScheme = new LayoutScheme("buttonbarexample",
                                  "rowspanbar", "mix1buttonbar");

// scheme for center/east arrangement
  LayoutScheme horizontalScheme = new LayoutScheme("buttonstackexample",
                                    "rowspanstack", "mix1buttonstack");

// current LayoutScheme, starting up with vertical 
  LayoutScheme currentScheme = verticalScheme;
  Container center;
  Container buttonBar;
  Container applicationForm;
  DataObject data;

  /**  SwitchLayoutWithBinding.
   */
  public SwitchFormLayoutWB() {
    bindActions();
  }

  public String getResource() {
    return currentScheme.layoutID;
  }

//---------------------------action: dynamically load center panel

  /** updateCenter.
   */
  public void ok() {
    toggleApplicationLayout();
  }

  public void copy() {
    System.out.println(data);
  }

  /** bind the button(s) to custom actions.
   */
  protected void bindActions() {
    ActionFactory.getInstance().registerHandler(Arrays.asList(new String[]{
      "ok",
      "copy" }), currentScheme.buttonPaneID, this);
  }

  protected void bind(JComponent comp) {
    data = new DataObject("Kleopatra", "+2-100", "+2-101");
    data.setNotes("nothing");
    binding = new BeanBinding(comp, data, true);
    binding.bindTextInput("name");
    binding.bindTextInput("phone");
    binding.bindTextInput("fax");
    binding.bindTextAreaInput("notes");
  }

//---------------------------helper
  protected void toggleApplicationLayout() {
    if (verticalScheme.equals(currentScheme)) {
      currentScheme = horizontalScheme;
    } else {
      currentScheme = verticalScheme;
    }
    rebuild();
  }

  protected void rebuild() {

    // re-arrange the sub-panels from the individual layout
    rebuild(center, currentScheme.centerPaneID);
    rebuild(buttonBar, currentScheme.buttonPaneID);

    // re-set the sub-panel ids as used in the application layout 
    buttonBar.setName(applicationButtonID);
    center.setName(applicationCenterID);

    // reload application layout
    rebuild(applicationForm, currentScheme.layoutID);
    applicationForm.validate();
    Window window = SwingUtilities.windowForComponent(applicationForm);
    window.pack();
  }

  /** re-arrange container's children according to resourceID.
   */
  private void rebuild(Container cont, String resourceID) {
    cont.setName(resourceID);
    FormFactory.getFormLoader().rebuild(cont, getClass());
  }

//---------------------------init ui
  protected JComponent buildForm(String resource) {

    // loading the sub-panels from the panel specific layout resources
    center = loadForm(currentScheme.centerPaneID);
    buttonBar = loadForm(currentScheme.buttonPaneID);

    // re-set the panel ids as used in the application layout resources
    center.setName(applicationCenterID);
    buttonBar.setName(applicationButtonID);

    // load the application layout resource 
    List comps = Arrays.asList(new Object[]{ center, buttonBar });
    applicationForm = FormFactory.getFormLoader().loadTemplate(resource,
        getClass(), comps);
    return (JComponent) applicationForm;
  }

  private Container loadForm(String resource) {
    return FormFactory.getFormLoader().load(resource, this.getClass());
  }

//------------------------ a convenience class for layout scheme info
  private static class LayoutScheme {

    public String layoutID;
    public String centerPaneID;
    public String buttonPaneID;
    public LayoutScheme(String layoutID, String centerPaneID,
        String buttonPaneID) {
      this.layoutID = layoutID;
      this.centerPaneID = centerPaneID;
      this.buttonPaneID = buttonPaneID;
    }
  }

//------------------------just some data bean

  /**
  */
  public class DataObject extends Model {

    private String fax;
    private String name;
    private String phone;
    private String notes;
    public DataObject(String name, String phone, String fax) {
      this.name = name;
      this.phone = phone;
      this.fax = fax;
    }

    private int count;
    public void setName(String name) {
      String oldName = getName();
      this.name = name;
      firePropertyChange("name", oldName, getName());
      setNotes("Name changed from " + oldName + " to " + getName());
    }

    public String getName() {
      return name;
    }

    public void setFax(String fax) {
      this.fax = fax;
    }

    public String getFax() {
      return fax;
    }

    public void setPhone(String phone) {
      this.phone = phone;
    }

    public String getPhone() {
      return phone;
    }

    public String getNotes() {
      return notes;
    }

    public void setNotes(String notes) {
      String oldNotes = getNotes();
      this.notes = notes;
      firePropertyChange("notes", oldNotes, getNotes());
    }

    public String toString() {
      return "Name/Phone/Fax/Notes: " + getName() + " / " + getPhone()
             + " / " + getFax() + " / " + getNotes();
    }
  }    // end inner class

//---------------------------Main

  /** main.
   * @param args
   */
  public static void main(String[] args) {
    Form form = new SwitchFormLayoutWB();
    FormUtils.createFrame(form, "Switch Layout", true);
  }
}
