/*
 * Copyright (c) 2002-2004 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  o Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  o Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  o Neither the name of JGoodies Karsten Lentzsch nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.kleopatra.forms.examples.bookmanager;

import java.util.List;

import de.kleopatra.forms.examples.data.Book;
import de.kleopatra.forms.examples.data.BookManager;
import de.kleopatra.forms.view.builder.Form;
import de.kleopatra.forms.view.builder.FormUtils;

/**
 * Builds a user interface for managing Books using a table to display
 * the Books and buttons to add, edit, and delete the selected book.
 * The models and Actions are provided by an underlying BookManagerModel. <p>
 * 
 * JW: copied and adjusted from JGoodies Binding Tutorial.
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.3 $
 *
 * @see BookManagerModel
 * @see com.jgoodies.binding.PresentationModel
 */
public final class BookManagerExample {

  /**
   * Provides a list of Books with selection and Action
   * for operating on the managed Books.
   */
  private final BookManagerModel bookManagerModel;

  // Launching **************************************************************
  public static void main(String[] args) {

    // set the LF
    System.getProperties().setProperty("swing.defaultlaf",
        "com.jgoodies.plaf.plastic.PlasticLookAndFeel");
    BookManagerExample example = new BookManagerExample();
    FormUtils.createFrame(example.getForm(), "BookManager", true);
  }

  private Form getForm() {
    return bookManagerModel.getForm();
  }

  // Instance Creation ******************************************************

  /**
   * Constructs a list editor using a example book list.
   */
  public BookManagerExample() {
    this(Book.BOOKS);
  }

  /**
   * Constructs a list editor for editing the given list of books.
   *
   * @param books   the list of books to edit
   */
  public BookManagerExample(List books) {
    this.bookManagerModel = new BookManagerModel(new BookManager(books));
  }
}
