/*
 * Created on 23.09.2004
 *
 */
package de.kleopatra.forms.examples.bookmanager;

import java.awt.event.*;

import javax.swing.*;

import com.jgoodies.binding.adapter.SingleListSelectionAdapter;
import com.jgoodies.binding.list.SelectionInList;

import de.kleopatra.forms.examples.data.TutorialUtils;
import de.kleopatra.forms.view.builder.AbstractForm;
import de.kleopatra.support.KleoUtilities;
import de.kleopatra.view.presentation.ActionFactory;

/**
 * Form representing BookManagerModel as a tabular view.
 * 
 * @author (C) 2004 Jeanette Winzenburg, Berlin
 * @version $Revision: 1.2 $ - $Date: 2004/10/01 10:22:42 $
 */
public class BookManagerForm extends AbstractForm {

  private final static String BOOK_MANAGER_RESOURCE = "booksmain";
  private static final String BOOKS_BUTTON_BAR = "booksbuttonbar";
  private SelectionInList bookSelection;
  private MouseListener doubleClickHandler;

//------------------------ constructor  
  public BookManagerForm(SelectionInList sil) {
    super(BOOK_MANAGER_RESOURCE, BookManagerForm.class);
    this.bookSelection = sil;
  }

//-------------------------- override/implement Form
  
  protected String getButtonResource() {
    return BOOKS_BUTTON_BAR;
  }

  protected void bind(JComponent form) {
    JTable bookTable = KleoUtilities.findTable(form, "books");
    bookTable.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
    bookTable.setModel(TutorialUtils.createBookTableModel(bookSelection));
    bookTable
      .setSelectionModel(new SingleListSelectionAdapter(bookSelection
        .getSelectionIndexHolder()));
    bookTable.addMouseListener(getDoubleClickHandler());
  }

  private MouseListener getDoubleClickHandler() {
    if (doubleClickHandler == null) {
      doubleClickHandler = new DoubleClickHandler("editBook");
    }
    return doubleClickHandler;
  }

  private class DoubleClickHandler extends MouseAdapter {

    private Object targetActionID;
    public DoubleClickHandler(String actionID) {
      this.targetActionID = actionID;
    }

    public void mouseClicked(MouseEvent e) {
      if (SwingUtilities.isLeftMouseButton(e) && (e.getClickCount() == 2)) {
        performAction();
      }
    }

    private void performAction() {
      Action action = ActionFactory.getInstance().getAction(targetActionID,
                        getButtonResource());
      if ((action != null) && (action.isEnabled())) {
        action.actionPerformed(null);
      }
    }
  }
}
