/*
 * Copyright (c) 2002-2004 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

package de.kleopatra.forms.examples.data;

import java.awt.Component;
import java.awt.Dimension;

import javax.swing.*;
import javax.swing.table.TableModel;

import com.jgoodies.binding.adapter.TableAdapter;


/**
 * Consists only of static methods that return instances
 * reused in multiple examples of the JGoodies Binding tutorial.<p>
 *
 * TODO: Add a Formatter for formatting and parsing cents. <p>
 *
 * JW: Copied from JGoodies Binding Tutorial. Added 
 * DataObjectListCellRenderer.
 * 
 * @author  Karsten Lentzsch
 * @version $Revision: 1.1 $
 */
public final class TutorialUtils {

  private TutorialUtils() {

    // Suppresses default constructor, ensuring non-instantiability.
  }

  /**
   * Creates and returns a renderer for books in a list.
   *
   * @return a renderer for books in lists.
   */
  public static ListCellRenderer createBookListCellRenderer() {
    return new BookListCellRenderer();
  }

  public static ListCellRenderer createDataObjectListCellRenderer() {
    return new DataObjectListCellRenderer();
  }
  /**
   * Creates and returns a TableModel for Books with columns
   * for the title and authors.
   *
   * @param listModel   the ListModel of Books to display in the table
   * @return a TableModel on the list of Books
   */
  public static TableModel createBookTableModel(ListModel listModel) {
    return new BookTableModel(listModel);
  }

  /**
   * Locates the given component on the screen's center.
   *
   * @param component   the component to be centered
   */
  public static void locateOnScreenCenter(Component component) {
    Dimension paneSize = component.getSize();
    Dimension screenSize = component.getToolkit().getScreenSize();
    component.setLocation((screenSize.width - paneSize.width) / 2,
        (screenSize.height - paneSize.height) / 2);
  }

  // Renderer ***************************************************************
  private static class BookListCellRenderer extends DefaultListCellRenderer {

    public Component getListCellRendererComponent(JList list, Object value,
        int index, boolean isSelected, boolean cellHasFocus) {
      Component component = super.getListCellRendererComponent(list, value,
                              index, isSelected, cellHasFocus);
      String text = null;
      if (value instanceof Book) {
        text = ((Book) value).getTitle();
      } else if (value instanceof MusicCD) {
        text = ((MusicCD) value).getTitle();
      }
      setText(" " + text);

      // Book book = (Book) value;
      // setText(" " + book.getTitle());
      return component;
    }
  }


  public static class DataObjectListCellRenderer extends DefaultListCellRenderer {

      public Component getListCellRendererComponent(JList list, Object value,
        int index, boolean isSelected, boolean cellHasFocus) {
        if (value instanceof DataObject) {
          value = ((DataObject) value).getName();
        }
        return super.getListCellRendererComponent(list, value, index, isSelected,
          cellHasFocus);
      
    }
}
  
  // TableModel *************************************************************
  private static class BookTableModel extends TableAdapter {

    private static final String[] COLUMNS = { "Title", "Authors", "Price" };
    private BookTableModel(ListModel listModel) {
      super(listModel, COLUMNS);
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
      Book book = (Book) getRow(rowIndex);
      switch (columnIndex) {

        case 0 :
          return book.getTitle();
        case 1 :
          return book.getAuthors();
        case 2 :
          return String.valueOf(book.getPrice());
        default :
          throw new IllegalStateException("Unknown column");
      }
    }
  }
}
