/*
 * Created on 26.09.2003
 *
 */
package custom.forms.io;

import java.io.*;
import java.util.*;

import org.jdom.*;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

import de.kleopatra.forms.description.*;
import de.kleopatra.forms.view.builder.FormDescription;
import de.kleopatra.forms.view.builder.FormFactory;
import de.kleopatra.persistence.IOHandler;
import de.kleopatra.persistence.ResourceManager;
import de.kleopatra.view.presentation.*;

/**
 *  just to demonstrate how to do this - c&p from FormReader3...
 *
 *	PENDING: does not handle borders
 *  PENDING: does not handle styled specs
 * 
 * @author Seessle
 * @version $Revision: 1.2 $ - $Date: 2004/09/30 10:16:40 $
 */
public abstract class AbstractJDomIOHandler implements IOHandler {

	private static final String USERDATA = "userData";
	private static final String COMPONENTCLASS = "componentClass";
	private static final String CONSTRAINT = "constraint";
	private static final String CONSTRAINTS = "constraints";
	private static final String COLUMN = "column";
	private static final String COLUMNS = "columns";
	private static final String GROUP = "group";
	private static final String ROW = "row";
	private static final String ROWS = "rows";
	private static final String TOOLTIP = "tooltip";
	private static final String MNEMONIC = "mnemonic";
	private static final String ICON = "icon";
	private static final String HELP = "help";
	private static final String ACCELERATOR = "accelerator";
	private static final String TEXT = "text";
	private static final String DESCRIPTION = "description";
	private static final String DESCRIPTIONS = "descriptions";
	private static final String ID = "id";
	private static final String PRESENTATION = "presentation";
	private static final String LAYOUT = "layout";
	private static final String FORM = "form";

//------------------------IOHandler
	public void write(Object object, String filename) {
		if (!checkType(object)) {
			return;
		}
		try {
			Document doc = new Document();
			Element e = new Element(FORM);
			genXml(object, e);
			doc.setRootElement(e);
			OutputStream out = new FileOutputStream(ensureExtension(filename));
			XMLOutputter outp = new XMLOutputter();
			outp.setTextTrim(true);
			outp.setIndent("  ");
			outp.setNewlines(true);
			outp.output(doc, out);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public Object read(String filename) {
		Object result = load(ensureExtension(filename));
		if (checkType(result)) {
			return result;
		}
		return null;
	}

	public Object readResource(String resourceName, Class targetClass) {
		InputStream in =
			ResourceManager.getInstance()
				.getResourceAsStream(ensureExtension(resourceName), targetClass);
		if (in != null) {
			Object result = load(in, resourceName);
			if (checkType(result)) {
				return result;
			}
		}
		return null;
	}

	public String ensureExtension(String filename) {
		filename = filename.trim();
		int lastPoint = getExtensionPosition(filename);
		if (lastPoint > 0) {
			filename = filename.substring(0, lastPoint);
		}
		return filename + getExtension();
	}

	public static Object load(String filename) {
		try {
			return load(new BufferedInputStream(new FileInputStream(filename)),
									filename);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public static Object load(InputStream in, String filename) {
		Object result = null;
		try {
			SAXBuilder builder = new SAXBuilder();
			builder.setIgnoringElementContentWhitespace(true);
			Document doc = builder.build(in);
			Element rootElement = doc.getRootElement();
			String rootName = rootElement.getName();
			if (rootName.equals(FORM)) {
				result = parseForm(rootElement, filename);
			} else if (rootName.equals(LAYOUT)) {
				result = parseLayout(rootElement);
			} else if (rootName.equals(PRESENTATION)) {
				result = parsePresentation(rootElement);
			} else {
				throw new RuntimeException("Unknown file");
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return result;
	}

//----------------------------helper to be implemented by subclasses

	/** checks and returns whether type of object is compatible with
	 *  getHandledType.
	 *
	 */
	protected abstract boolean checkType(Object object);

//-----------------------------helper
	protected int getExtensionPosition(String filename) {
		return filename.lastIndexOf(".");
	}

//-----------------------------helper jdom

	/** subs will delegate to gen<type>XML.
	 *  PRE: checkType(object)
	 */
	protected abstract void genXml(Object object, Element form);

	/**
	 * @param formDesc
	 * @param e
	 */
	protected void genFormXml(FormDescription formDesc, Element form) {
		form.addContent(new Element(ID).setText(formDesc.getFormID().toString()));
		Element layout = new Element(LAYOUT);
		genLayoutXml(formDesc.getLayoutDescription(), layout);
		form.addContent(layout);
		Element presentation = new Element(PRESENTATION);
		genPresentationXml(formDesc.getPresentation(), presentation);
		form.addContent(presentation);
	}

	/**
	 * @param presentation
	 * @param presentation2
	 */
	protected void genPresentationXml(Presentation presentation,
			Element presentationElement) {

		presentationElement
			.addContent(new Element(ID)
				.setText(presentation.getPresentationID().toString()));
		Element descriptionsElement = new Element(DESCRIPTIONS);
		Iterator descriptions =
			presentation.getPresentationDescriptions().iterator();
		while (descriptions.hasNext()) {
			PresentationDescription description =
				(PresentationDescription) descriptions.next();
			Element descriptionElement = new Element(DESCRIPTION);
			descriptionElement.setAttribute(ID,
					description.getPresentationID().toString());
			descriptionElement.setAttribute(TEXT, description.getText());
			if (description.getAccelerator() != null) {
				descriptionElement.setAttribute(ACCELERATOR,
						description.getAccelerator());
			}
			if (description.getHelpText() != null) {
				descriptionElement.setAttribute(HELP, description.getHelpText());
			}
			if (description.getIcon() != null) {
				descriptionElement.setAttribute(ICON, description.getIcon());
			}
			if (description.getMnemonic() != null) {
				descriptionElement.setAttribute(MNEMONIC, description.getMnemonic());
			}
			if (description.getToolTipText() != null) {
				descriptionElement.setAttribute(TOOLTIP,
						description.getToolTipText());
			}
			descriptionsElement.addContent(descriptionElement);
		}
		presentationElement.addContent(descriptionsElement);
	}

	/**
	 * @param description
	 * @param layout
	 */
	protected void genLayoutXml(LayoutDescription description, Element layout) {
		genLineXml(description.getRows().iterator(), layout, ROWS, ROW);
		genLineXml(description.getColumns().iterator(), layout, COLUMNS, COLUMN);
		Iterator constraints = description.getConstraints().iterator();
		Element constraintsElement = new Element(CONSTRAINTS);
		while (constraints.hasNext()) {
			Constraint constraint = (Constraint) constraints.next();
			Element constraintElement = new Element(CONSTRAINT);
			constraintElement.setAttribute(ID,
					constraint.getComponentID().toString());
			constraintElement.setAttribute(COMPONENTCLASS,
					constraint.getComponentClassName());
			constraintElement.setAttribute(LAYOUT, constraint.getEncodedString());

			/*    if (constraint.getUserData() != null) {
						constraintElement.addContent(
							(Element) constraint.getUserData());
					}
			*/
			constraintsElement.addContent(constraintElement);
		}
		layout.addContent(constraintsElement);
	}

	protected void genLineXml(Iterator lines, Element layout, String groupName,
			String elementName) {
		Element linesElement = new Element(groupName);
		while (lines.hasNext()) {
			Line line = (Line) lines.next();
			Element lineElement = new Element(elementName);
			lineElement.setAttribute(LAYOUT, line.getValidEncoding());
			if (line.getGroupID() != null) {
				lineElement.setAttribute(GROUP, line.getGroupID().toString());
			}
			linesElement.addContent(lineElement);
		}
		layout.addContent(linesElement);
	}

/**
 * @param rootElement
 * @return
 */
	private static Object parseForm(Element formElement, String filename) {

		FormDescription form = FormFactory.createFormDescription();
		form.setFormID(formElement.getChild(ID).getText());
		if (formElement.getChild(LAYOUT) != null) {
			LayoutDescription layout = parseLayout(formElement.getChild(LAYOUT));
			form.setLayoutDescription(layout);
		}
		if (formElement.getChild("presentation") != null) {
			Presentation presentation =
				parsePresentation(formElement.getChild("presentation"));
			form.setPresentation(presentation);
		}

/*
		ResourceBundle bundle = ResourceBundle.getBundle("np-form2");
		Vector entries = new Vector();
		Enumeration keys = bundle.getKeys();
		while (keys.hasMoreElements()) {
			String key = (String) keys.nextElement();
			if (key.endsWith(".id")) {
				entries.add(key.substring(0, key.length() - ".id".length()));
			}
		}
		DefaultPresentation presentation = new DefaultPresentation();
		Vector descriptions = new Vector();
		for (Iterator iter = entries.iterator(); iter.hasNext(); ) {
			String key = (String) iter.next();
			PresentationDescription desc = new DefaultPresentationDescription();
			desc.setPresentationID(bundle.getString(key + ".id"));
			desc.setText(bundle.getString(key + ".text"));
			descriptions.add(desc);
		}
		presentation.setPresentationDescriptions(descriptions);
		form.setPresentation(presentation);
		try {
			Properties props = new Properties();
			FormIO formIO = new FormIO();
			props.load(new FileInputStream(formIO.ensureExtension(filename,
					".properties")));
			Vector entries = new Vector();
			Enumeration keys = props.keys();
			while (keys.hasMoreElements()) {
				String key = (String) keys.nextElement();
				if (key.endsWith(".id")) {
					entries.add(key.substring(0, key.length() - ".id".length()));
				}
			}
			DefaultPresentation presentation = new DefaultPresentation();
			Vector descriptions = new Vector();
			for (Iterator iter = entries.iterator(); iter.hasNext(); ) {
				String key = (String) iter.next();
				PresentationDescription desc = new DefaultPresentationDescription();
				desc.setPresentationID(props.getProperty(key + ".id"));
				desc.setText(props.getProperty(key + ".text"));
				descriptions.add(desc);
			}
			presentation.setPresentationDescriptions(descriptions);
			form.setPresentation(presentation);
		} catch (FileNotFoundException e) {

			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {

			// TODO Auto-generated catch block
			e.printStackTrace();
		}
*/
		return form;
	}

/**
 * @param rootElement
 * @return
 */
	protected static Presentation parsePresentation(
			Element presentationElement) {

		DefaultPresentation presentation = new DefaultPresentation();
		Element descriptionsElement = presentationElement.getChild(DESCRIPTIONS);
		List descriptionList = descriptionsElement.getChildren(DESCRIPTION);
		Iterator descriptions = descriptionList.iterator();
		Vector parsedDescriptions = new Vector();
		while (descriptions.hasNext()) {
			Element descriptionElement = (Element) descriptions.next();
			DefaultPresentationDescription description =
				new DefaultPresentationDescription();
			description.setPresentationID(descriptionElement.getAttributeValue(ID));
			description.setText(descriptionElement.getAttributeValue(TEXT));
			if (descriptionElement.getAttributeValue(ACCELERATOR) != null) {
				description
					.setAccelerator(descriptionElement.getAttributeValue(ACCELERATOR));
			}
			if (descriptionElement.getAttributeValue(HELP) != null) {
				description.setHelpText(descriptionElement.getAttributeValue(HELP));
			}
			if (descriptionElement.getAttributeValue(ICON) != null) {
				description.setIcon(descriptionElement.getAttributeValue(ICON));
			}
			if (descriptionElement.getAttributeValue(MNEMONIC) != null) {
				description
					.setMnemonic(descriptionElement.getAttributeValue(MNEMONIC));
			}
			if (descriptionElement.getAttributeValue(TOOLTIP) != null) {
				description
					.setToolTipText(descriptionElement.getAttributeValue(TOOLTIP));
			}
			parsedDescriptions.add(description);
		}
		presentation.setPresentationDescriptions(parsedDescriptions);
		return presentation;
	}

/**
 * @param element
 * @return
 */
	protected static LayoutDescription parseLayout(Element layoutElement) {

		DefaultLayoutDescription layout = new DefaultLayoutDescription();

		//layout.setLayoutID(layoutElement.getChild("id").getText());
		LayoutFactory factory = LayoutBuilderFactory.getLayoutFactory();
		Element linesElement = layoutElement.getChild(ROWS);
		List lineList = linesElement.getChildren(ROW);
		Iterator lines = lineList.iterator();
		List parsedLines = new Vector();
		while (lines.hasNext()) {
			Element lineElement = (Element) lines.next();
			Row row = factory.createRow(lineElement.getAttributeValue(LAYOUT));
	//		row.setEncodedString();
			Attribute groupAttribute = lineElement.getAttribute(GROUP);
			if (groupAttribute != null) {
				row.setGroupID(groupAttribute.getValue());
			}
			parsedLines.add(row);
		}
		layout.setRows(parsedLines);
		Element columnsElement = layoutElement.getChild(COLUMNS);
		List columnList = columnsElement.getChildren(COLUMN);
		Iterator columns = columnList.iterator();
		List parsedColumns = new Vector();
		while (columns.hasNext()) {
			Element columnElement = (Element) columns.next();
			Column column = factory.createColumn(columnElement.getAttributeValue(LAYOUT));
//			column.setEncodedString();
			Attribute groupAttribute = columnElement.getAttribute(GROUP);
			if (groupAttribute != null) {
				column.setGroupID(groupAttribute.getValue());
			}
			parsedColumns.add(column);
		}
		layout.setColumns(parsedColumns);
		Element constraintsElement = layoutElement.getChild(CONSTRAINTS);
		List constraintList = constraintsElement.getChildren(CONSTRAINT);
		Iterator constraints = constraintList.iterator();
		List parsedConstraints = new Vector();
		while (constraints.hasNext()) {
			Element constraintElement = (Element) constraints.next();
			Constraint constraint =
				factory.createConstraint(constraintElement.getAttributeValue(LAYOUT));
			String compClass = constraintElement.getAttributeValue("class");
			if (compClass == null) {
				compClass = constraintElement.getAttributeValue(COMPONENTCLASS);
			}
			constraint.setComponentClassName(compClass);
			constraint.setComponentID(constraintElement.getAttributeValue(ID));
			Element userData = constraintElement.getChild(USERDATA);
			if (userData != null) {
				Element udClone = (Element) userData.clone();

				//  constraint.setUserData(udClone);
			}
			parsedConstraints.add(constraint);
		}
		layout.setListConstraints(parsedConstraints);
		return layout;
	}
}
