/*
 * Created on 09.09.2003
 *
 */
package custom.forms.view;

import java.awt.Color;
import java.awt.Graphics;
import java.util.Set;

import javax.swing.JLabel;

import de.kleopatra.view.UIProperties;
import de.kleopatra.view.factory.SwingComponentTypeMap;
import de.kleopatra.view.factory.SwingComponentTypeMap.Instantiator;

/**
 * Sample custom ComponentTypeMap that demonstrates several 
 * possibilities to support mapping of custom types.
 * 
 * 
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.2 $ - $Date: 2004/10/04 12:18:34 $
 */
public class ExSwingComponentTypeMap extends SwingComponentTypeMap {

	/** make sure to return types created by custom mechanism.
	 *  super will return all generic types registered via
	 *  via addComponentType and addComponentInstantiator. 
	 */
	public Set getComponentTypes() {
		Set set = createSet();
		set.addAll(super.getComponentTypes());
		set.add("TestComponent.red");
		return set;
	}
	/** support mapping by custom mechanism.
	 * 
	 */
	protected Object factoryHook(Object componentType) {
	  // direct instantiation of custom type
		if ("TestComponent.red".equals(componentType)) {
				return new TestComponentRed();
		}
		return super.factoryHook(componentType);
	}

	protected void initComponentTypes() {
		super.initComponentTypes();
		// add  mapping by classname
		addComponentType("ButtonedText", "custom.forms.view.JButtonedText");
		// add mapping by class
		addComponentType("TextFieldHighlighted", JExtraTextField.class);
	}

	protected void initInstantiatorTypes() {
		super.initInstantiatorTypes();
		// add mapping by Instantiator
		addComponentInstantiator("TestComponent.blue", new Instantiator() {

			public Object instantiate() {
				return new TestComponent();
			}
			
		});
		// add a label type which should not be 
		// customized with a auto-colon.
    addComponentInstantiator("Label.explaining", new Instantiator() {

      public Object instantiate() {
        JLabel label = new JLabel();
        label.putClientProperty(UIProperties.NO_LABELENDING, Boolean.TRUE);
        return label;
      }

    });
		
	}
	public static class TestComponentRed extends TestComponent {
		public void paint(Graphics g) {
			g.setColor(Color.red);
			g.fillRect(0, 0, getWidth(), getHeight());
		}

	}
	
}
