/*
 * Created on 05.09.2003
 */
package custom.forms.view.builder;

import java.awt.Container;
import java.util.Locale;
import java.util.Vector;

import de.kleopatra.forms.view.builder.FormDescription;
import de.kleopatra.forms.view.builder.FormWriter;
import de.kleopatra.view.presentation.Presentation;

/**
 * @author Seessle
 */
public class I15dPresentationFormWriter extends FormWriter {

	public Presentation getPresentation(String resourceName,
			Class targetClass) {
		Presentation presentation = null;
		Vector names = calculateBundleNames(resourceName, Locale.getDefault());
		for (int i = names.size() - 1; (presentation == null) && (i >= 0); i--) {
			presentation = getFormIO().getPresentation((String) names.get(i),
					targetClass);
		}
		return presentation;
	}

	public Container buildForm(String descriptionName, Class targetClass,
			boolean customize) {
		FormDescription formDescription =
			getFormIO().getFormDescription(descriptionName, targetClass);
		if (formDescription != null) {
			return buildForm(descriptionName, targetClass, formDescription,
					customize);
		}
		return null;
	}

	/**
	 * @param descriptionName
	 * @param formDescription
	 * @param customize
	 * @return
	 */
	private Container buildForm(String descriptionName, Class targetClass,
			FormDescription formDescription, boolean customize) {
		Presentation presentation = getPresentation(descriptionName, targetClass);
		if (presentation != null) {
			formDescription.setPresentation(presentation);
		}
		return buildForm(formDescription, customize);
	}

	private static Vector calculateBundleNames(String baseName, Locale locale) {

		final Vector result = new Vector();
		final String language = locale.getLanguage();
		final int languageLength = language.length();
		final String country = locale.getCountry();
		final int countryLength = country.length();
		final String variant = locale.getVariant();
		final int variantLength = variant.length();
		result.addElement(baseName);
		if (languageLength + countryLength + variantLength == 0) {

			//The locale is "", "", "".
			return result;
		}
		final StringBuffer temp = new StringBuffer(baseName);
		temp.append('_');
		temp.append(language);
		if (languageLength > 0) {
			result.addElement(temp.toString());
		}
		if (countryLength + variantLength == 0) {
			return result;
		}
		temp.append('_');
		temp.append(country);
		if (countryLength > 0) {
			result.addElement(temp.toString());
		}
		if (variantLength == 0) {
			return result;
		}
		temp.append('_');
		temp.append(variant);
		result.addElement(temp.toString());
		return result;
	}

	public static void main(String[] args) {
		Vector v = calculateBundleNames("foo", new Locale("de", "DE", "EUR"));
		System.out.println(v);
	}
}
