package de.kleopatra.forms.examples;

import java.awt.*;
import java.io.File;
import java.util.Arrays;

import javax.swing.*;

import de.kleopatra.forms.view.builder.*;
import de.kleopatra.persistence.ResourceManager;
import de.kleopatra.view.JTitledBorderPanel;
import de.kleopatra.view.presentation.ActionFactory;

/** Example how to load resources manually, either by lookup
 *  from classpath or from file.
 *
 *  @author (c) Jeanette Winzenburg, Berlin
 *  @version $Revision: 1.4 $ $Date: 2004/09/30 10:16:41 $
 *
 */
public class LoadFromFile {

  protected JFrame frame;
  protected FormIOHelper formIOHelper;
  protected Component center;
  protected String buttonBarID = "buttonbar";

  /**
   */
  public LoadFromFile() {
    frame = new JFrame("PlaceHolderUsage");
    JOptionPane.setRootFrame(frame);
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    center = buildCenterPanel();
    frame.getContentPane().add(center);
    frame.getContentPane().add(buildButtonPanel(), BorderLayout.SOUTH);
    frame.pack();
    frame.setLocationRelativeTo(null);
    frame.show();
  }

//---------------------------action callback
  String[] actionIDs = { "updateAutoReplace" };

  /**
   * choose a file, read the contained layout resource, load
   *  a container from it and replace the center pane with it.
   */
  public void updateAutoReplace() {
    Container container = loadFromFile(readFilename());
    if (container == null) {
      return;
    }
    replaceCenter(container);
  }

  protected Container loadFromFile(String filename) {
    if (filename == null) {
      return null;
    }
    return getFormLoader().loadFromFile(filename);
  }

  /** load description resources from classpath.
   */
  protected Container loadFromResource(String resourceID) {
    return getFormLoader().load(resourceID, getClass());
  }

  /** bind the button(s) to custom actions.
   */
  protected void registerActions() {
    ActionFactory.getInstance().registerHandler(Arrays.asList(actionIDs),
        buttonBarID, this);
  }

//---------------------------helper
  protected void replaceCenter(Component newCenter) {
    frame.getContentPane().remove(center);
    center = newCenter;
    frame.getContentPane().add(center);
    frame.pack();
  }

  protected String readFilename() {
    String filename = getFormIOHelper().chooseFormDescriptionFilename(true);
    if (filename != null) {
      File file = new File(filename);

      // make sure the ResourceManager is pointing to the correct directory
      // outer and placehoder have to be in the same directory for auto-replace
      // to work
      ResourceManager.getInstance().setUserResourceDir(file.getParent());
    }
    return filename;
  }

//---------------------------init ui
  protected Container buildCenterPanel() {
    JComponent form = new JTitledBorderPanel("Some Panel to replace...");
    ((JComponent) form).setPreferredSize(new Dimension(200, 200));
    return form;
  }

  protected Container buildButtonPanel() {
    Container container = loadFromResource(buttonBarID);
    registerActions();
    return container;
  }

//---------------------------factory methods

  /** need to use an extended FormLoader to cope with file reads.
   *
   */
  protected ExtFormLoader getFormLoader() {
    return FormFactory.createExtFormLoader();
  }

  protected FormIOHelper getFormIOHelper() {
    if (formIOHelper == null) {
      formIOHelper = new FormIOHelper();
      formIOHelper.setOwner(JOptionPane.getRootFrame());
    }
    return formIOHelper;
  }

//---------------------------Main
  public static void main(String[] args) {
    new LoadFromFile();
  }
}
