package de.kleopatra.forms.examples;

import java.util.Arrays;
import java.util.Locale;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import de.kleopatra.forms.view.builder.*;
import de.kleopatra.persistence.ResourceKeys;
import de.kleopatra.support.KleoUtilities;
import de.kleopatra.view.UIProperties;

/** Example how to
 *
 *  <ul>
 *  <li> use placeholders and re-usable resources
 *  <li> use custom widgets
 *  <li> enable auto-colon label
 *  <li> replace components dynamically based on some other's value.
 *  <li> use an application-wide ResourceBundle
 *  </ul>
 *
 *  @author (c) 2003, 2004 Jeanette Winzenburg, Berlin
 *  @version $Revision: 1.5 $ $Date: 2004/10/04 12:18:34 $
 *
 */
public class NovaExample extends AbstractForm {

  private static final String NOVA = "nova";
  private static final String NOVA_BUTTON = "novabutton";
  private static final String TRIGGER = "postalCode";
  private static final String TRIGGER_TARGET = "region";
  private JComboBox regionsChoice;
  private JTextField region;

  /**
   */
  public NovaExample() {
    registerActions();
  }

  public String getResource() {
    return NOVA;
  }

  protected String getButtonResource() {
    return NOVA_BUTTON;
  }

//---------------------------action: dynamically load center panel
  String[] actionIDs = { "commit", "exit" };

  /** some fake commit
   */
  public void commit() {
    System.out.println("committed");
  }

  /** some fake exit
   */
  public void exit() {
    System.out.println("exited");
  }

  /** bind the button(s) to custom actions.
   */
  protected void registerActions() {

    // the following line is necessary if you want to re-use
    // the button panel resource in a different container hierarchy: 
    // the loading process registers default delegate
    // actions for the button panel under the name of the 
    // button panel resource. So we have to move them
    // to our own realm of responsibility
    // a bit clumsy still... will change...
    // ActionFactory.getInstance().moveActions(NOVA_BUTTON, NOVA);
    registerHandler(Arrays.asList(actionIDs), this);
  }

//---------------------------helper
  protected void updateRegion(JTextField field) {

    // just to show the principle....
    String code = field.getText();
    if ((code.equals("valid") && (region != null)
        && (region.getParent() == null))) {
      region.setText("validRegion");
      FormFactory.getFormLoader().replaceComponent(getFormComponent(),
          getClass(), region);
      field.requestFocus();
    } else if ((!code.equals("valid")) && (regionsChoice != null)
               && (regionsChoice.getParent() == null)) {
      FormFactory.getFormLoader().replaceComponent(getFormComponent(),
          getClass(), regionsChoice);
      getFormComponent().repaint();
      field.requestFocus();
    }
  }

//---------------------------init ui
  protected void bind(JComponent form) {
    installComponents(form);
    installListeners(form);
  }

  private void installComponents(JComponent form) {
    regionsChoice = KleoUtilities.findComboBox(form, TRIGGER_TARGET);
    if (regionsChoice != null) {
      region = new JTextField();
      region.setName(TRIGGER_TARGET);
      region.setEditable(false);
    }
  }

  private void installListeners(JComponent form) {
    JComponent postalComp = KleoUtilities.findJChildByName(form, TRIGGER);
    if (postalComp instanceof JTextField) {
      installPostalCodeListener((JTextField) postalComp);
    }
  }

  private void installPostalCodeListener(final JTextField field) {
    field.getDocument().addDocumentListener(new DocumentListener() {

      public void changedUpdate(DocumentEvent e) {}

      public void insertUpdate(DocumentEvent e) {
        updateRegion(field);
      }

      public void removeUpdate(DocumentEvent e) {
        updateRegion(field);
      }
    });
  }

//---------------------------Main
  private static void configureUIProperties() {

    // set the LF
    System.getProperties().setProperty("swing.defaultlaf",
        "com.jgoodies.plaf.plastic.PlasticLookAndFeel");

    // use the custom (extended) componentTypeMap
    System.getProperties().setProperty(ResourceKeys.COMPONENT_MAP_KEY,
        "custom.forms.view.ExSwingComponentTypeMap");
    // use the custom (extended) customizerMap to enable
    // auto-colon label
    System.getProperties().setProperty(ResourceKeys.CUSTOMIZER_MAP_KEY,
        "custom.forms.view.ExSwingCustomizerMap");

    // set custom locale (base is Esperanto)
    Locale.setDefault(Locale.ENGLISH);
    UIManager.put(UIProperties.COMMON_RESOURCE_BUNDLE_NAME, NOVA);
  }

  public static void main(String[] args) {
    configureUIProperties();
    Form novaExample = new NovaExample();
    FormUtils.createFrame(novaExample, "Nova Example", true);
  }
}
