package de.kleopatra.forms.examples;

import java.awt.Container;
import java.awt.Window;
import java.util.Arrays;

import javax.swing.*;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import de.kleopatra.forms.view.builder.*;
import de.kleopatra.support.KleoUtilities;

/** Example how to change resources of a part of the form.
 *
 *  demonstrates:
 *
 *  <ul>
 *  <li> how to load an ui on startup from a resource relative
 *      to the client. Note that the quickstartd actually is
 *      a placeholder that's automatically replaced.
 *  <li> how to bind the an action to a method of the client
 *  <li> how to find components by name, f.i. to customize them or
 *    to register listeners
 *  <li> how to replace one component with another programmatically
 *  <li> "quickstartoverviewb" shows the limits of automatically
 *    replacing placeholders: supported for direct children only.
 *  </ul>
 *
 *  @version $Revision: 1.3 $ $Date: 2004/09/30 10:16:41 $
 *  @author (C) 2003 Jeanette Winzenburg
 *
 */
public class QuickStart extends AbstractForm {

  private static final String QUICKSTART_RESOURCE = "quickstart";
  protected String centerID = "quickstartd";
  protected String buttonBarID = "pack";
  protected String listID = "samples";
  protected String[] resources = { "quickstartd", "quickstartc",
                                   "quickstartb", "quickstartpropellerbox",
                                   "quickstartgeneralbox", "quickstarta",
                                   "quickstartoverview",
                                   "quickstartoverviewb" };
  public QuickStart() {
    registerActions();
  }

  public String getResource() {
    return QUICKSTART_RESOURCE;
  }

//---------------------------action: dynamically load center panel

  /** updateCenter.
   */
  public void pack() {
    getWindow().pack();
  }

//---------------------------list selectionListener
  private ListSelectionListener createListSelectionListener() {
    ListSelectionListener l = new ListSelectionListener() {

      public void valueChanged(ListSelectionEvent e) {
        if (e.getValueIsAdjusting()) {
          return;
        }
        JList list = (JList) e.getSource();
        if (list.isSelectionEmpty()) {
          return;
        }
        updateCenter(String.valueOf(list.getSelectedValue()));
      }
    };
    return l;
  }

  protected void updateCenter(String resourceName) {
    Container container = FormFactory.getFormLoader().load(resourceName,
                            getClass());
    if (container == null) {
      return;
    }
    container.setName(centerID);
    customizeBorder(container);
    FormFactory.getFormLoader().replaceComponent(getFormComponent(), getClass(), container);
  }

//---------------------------init ui
  protected void bind(JComponent form) {
    bindComponents(form);
    customizeBorder(form);
  }

  protected void registerActions() {
    registerHandler(Arrays.asList(new String[]{ "pack" }), this);
  }

  protected void bindComponents(Container container) {
    JList list = KleoUtilities.findList(container, listID);
    if (list == null) {
      return;
    }
    list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    list.setListData(resources);
    list.setSelectedIndex(0);
    list.addListSelectionListener(createListSelectionListener());
  }

  //---------------------------helper
  private void customizeBorder(Container container) {
    JComponent center = KleoUtilities.findJComponentByName(container,
                          centerID);
    if (center == null) {
      return;
    }
    Border border = center.getBorder();
    Border centerBorder =
      BorderFactory.createBevelBorder(BevelBorder.LOWERED);
    if (border != null) {
      centerBorder = BorderFactory.createCompoundBorder(centerBorder, border);
    }
    center.setBorder(centerBorder);
  }

  private Window getWindow() {
    Window window = SwingUtilities.windowForComponent(form);
    return window;
  }

//---------------------------Main
  public static void main(String[] args) {
    // set the LF
    System.getProperties().setProperty("swing.defaultlaf",
        "com.jgoodies.plaf.plastic.PlasticLookAndFeel");

    Form form = new QuickStart();
    FormUtils.createFrame(form, "QuickStart Design Alternatives",  true);
  }
}
