package de.kleopatra.forms.examples;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.util.Arrays;

import javax.swing.*;

import de.kleopatra.forms.view.builder.FormFactory;
import de.kleopatra.support.KleoUtilities;
import de.kleopatra.view.JTitledBorderPanel;
import de.kleopatra.view.presentation.ActionFactory;

/** Example how to use a pre-built layout resource.
 *
 *  demonstrates:
 *
 *  <ul>
 *  <li> how to load an ui on startup from a resource relative
 *      to the client
 *  <li> how to bind the default action to a method of the client.
 *  </ul>
 *
 *  @version $Revision: 1.2 $ $Date: 2004/09/30 10:16:41 $
 *  @author (C) 2003, 2004 Jeanette Winzenburg
 *
 */
public class SimpleLoading {

  protected JFrame frame;
  private static final String SIMPLE_FORM_RESOURCE = "simpleform";

  /**  SimpleForm.
   */
  public SimpleLoading() {
    frame = new JFrame("Simple Form");
    JOptionPane.setRootFrame(frame);
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

    // registering actions can be done before or after loading the form
    registerActions();
    frame.getContentPane().add(buildForm());

//    registerActions();
    frame.pack();
    frame.setLocationRelativeTo(null);
    frame.show();
  }

//---------------------------action handling
  String[] actionIDs = { "someAction" };

  /** do something.
   *  NOTE: the methodName has to be exactly the same as
   *    the constraintID = actionCommand of the button. <p>
   * NOTE: both handling class and action method need to
   *  be publicly accessible <p>
   *
   */
  public void someAction() {
    log("executed someAction method");
  }

  /** Every AbstractButton has a default action set,
   *  customized and registered with the
   *  ActionFactory. A Client can registers itself
   *  as an eventHandler for the action s/he is
   *  interested in or register a custom action as handler.
   *
   */
  protected void registerActions() {
    ActionFactory.getInstance().registerHandler(Arrays.asList(actionIDs),
        SIMPLE_FORM_RESOURCE, this);
    ActionFactory.getInstance().registerActionAsHandler("otherAction",
        SIMPLE_FORM_RESOURCE, new AbstractAction() {

      public void actionPerformed(ActionEvent e) {
        log("performed otherAction " + e);
      }
    });
  }

  protected void log(String text) {
    JTextArea area = KleoUtilities.findTextArea(frame, "notes");
    area.append(text + "\n");
  }

//---------------------------init ui
  protected Container buildForm() {
    Container form = FormFactory.getFormLoader().load(SIMPLE_FORM_RESOURCE,
                       getClass());
    if (form == null) {
      form = new JTitledBorderPanel("resource not found: "
          + SIMPLE_FORM_RESOURCE);
      ((JComponent) form).setPreferredSize(new Dimension(300, 200));
    }
    return form;
  }

//---------------------------Main
  public static void main(String[] args) {
    new SimpleLoading();
  }
}
