package de.kleopatra.forms.examples;

import java.awt.event.ActionEvent;
import java.util.Arrays;

import javax.swing.*;

import de.kleopatra.forms.view.builder.*;
import de.kleopatra.support.KleoUtilities;

/** Example how to use a pre-built layout resource. <p>
 *
 *  Similar to SimpleLoading, simplifies code by 
 *  extending AbstractForm. <p>
 *
 *  @version $Revision: 1.1 $ $Date: 2004/10/01 13:38:07 $
 *  @author (C) 2003, 2004 Jeanette Winzenburg
 *
 */
public class SimpleLoadingForm extends AbstractForm {

  private static final String SIMPLE_FORM_RESOURCE = "simpleform";

  /**  SimpleForm.
   */
  public SimpleLoadingForm() {
    registerActions();
  }

//---------------------- implement/override Form/AbstractForm
  public String getResource() {
    return SIMPLE_FORM_RESOURCE;
  }

  /** do nothing, no binding. */
  protected void bind(JComponent form) {}

//---------------------------action handling
  String[] actionIDs = { "someAction" };

  /** do something.
   *  NOTE: the methodName has to be exactly the same as
   *    the constraintID = actionCommand of the button. <p>
   * NOTE: both handling class and action method need to
   *  be publicly accessible <p>
   *
   */
  public void someAction() {
    log("executed someAction method");
  }

  /** Every AbstractButton has a default action set,
   *  customized and registered with the
   *  ActionFactory. A Client can registers itself
   *  as an eventHandler for the action s/he is
   *  interested in or register a custom action as handler.
   *
   */
  protected void registerActions() {
    registerHandler(Arrays.asList(actionIDs), this);
    registerActionAsHandler("otherAction", new AbstractAction() {

      public void actionPerformed(ActionEvent e) {
        log("performed otherAction " + e);
      }
    });
  }

  protected void log(String text) {
    JTextArea area = KleoUtilities.findTextArea(getFormComponent(), "notes");
    area.append(text + "\n");
  }

//---------------------------Main
  public static void main(String[] args) {
    Form form = new SimpleLoadingForm();
    FormUtils.createFrame(form, true);
  }
}
